package com.atolcd.repo.action.executer;

import java.io.File;
import java.io.FileOutputStream;
import java.util.List;

import org.alfresco.model.ContentModel;
import org.alfresco.repo.action.ParameterDefinitionImpl;
import org.alfresco.repo.action.executer.ActionExecuterAbstractBase;
import org.alfresco.repo.content.MimetypeMap;
import org.alfresco.service.cmr.action.Action;
import org.alfresco.service.cmr.action.ParameterDefinition;
import org.alfresco.service.cmr.dictionary.DataTypeDefinition;
import org.alfresco.service.cmr.repository.ContentReader;
import org.alfresco.service.cmr.repository.ContentService;
import org.alfresco.service.cmr.repository.ContentWriter;
import org.alfresco.service.cmr.repository.NodeRef;
import org.alfresco.util.TempFileProvider;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.util.Assert;

import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfReader;
import com.lowagie.text.pdf.PdfStamper;

public class PDFWatermarkActionExecuter extends ActionExecuterAbstractBase
	implements InitializingBean {

    public static final String NAME = "watermark";
    public static final String PARAM_TEXT = "watermark-text";

    private ContentService contentService;

    public void setContentService(ContentService contentService) {
	this.contentService = contentService;
    }

    public void afterPropertiesSet() throws Exception {
	Assert.notNull(contentService);
    }

    @Override
    protected void executeImpl(Action action, NodeRef actionedUponNodeRef) {
	ContentReader reader = contentService.getReader(actionedUponNodeRef,
		ContentModel.PROP_CONTENT);

	if (MimetypeMap.MIMETYPE_PDF.equals(reader.getMimetype())) {
	    PdfReader pdfReader = null;
	    PdfStamper stamper = null;
	    try {
		pdfReader = new PdfReader(reader.getContentInputStream());
		File newPdf = TempFileProvider.createTempFile("tmp",
			"watermark");
		FileOutputStream fos = new FileOutputStream(newPdf);
		stamper = new PdfStamper(pdfReader, fos);
		BaseFont bf = BaseFont.createFont(BaseFont.HELVETICA,
			BaseFont.WINANSI, BaseFont.NOT_EMBEDDED);
		PdfContentByte under;
		int total = pdfReader.getNumberOfPages() + 1;
		for (int i = 1; i < total; i++) {
		    under = stamper.getUnderContent(i);
		    under.beginText();
		    under.setFontAndSize(bf, 18);
		    under.setTextMatrix(0, 0.5f, -0.5f, 0, 30, 30);
		    under.showText((String) action
			    .getParameterValue(PARAM_TEXT));
		    under.endText();
		}
		stamper.close();

		ContentWriter writer = contentService.getWriter(
			actionedUponNodeRef, ContentModel.PROP_CONTENT, true);
		writer.putContent(newPdf);
	    } catch (Exception e) {
		e.printStackTrace();
	    }
	}
    }

    @Override
    protected void addParameterDefinitions(List<ParameterDefinition> paramList) {
	paramList
		.add(new ParameterDefinitionImpl(PARAM_TEXT,
			DataTypeDefinition.TEXT, true,
			getParamDisplayLabel(PARAM_TEXT)));
    }
}
